@@ -3,11 +3,7 @@ QT=~/Qt/6.7.2/macos
 mkdir _bin
 
 pushd filetype
-rm -fr build
-mkdir build
-cd build
-${QT}/bin/qmake "CONFIG+=release" ../libfiletype.pro
-make -j4
+bash build-gcc.sh
 popd
 
 pushd packaging/mac
@@ -24,6 +20,6 @@ popd
 mv build/Guitar.app .
 rm -f Guitar-macos.zip
 zip -r Guitar-macos.zip Guitar.app
-curl -T Guitar-macos.zip ftp://192.168.0.5/Public/pub/nightlybuild/
+curl -T Guitar-macos.zip ftp://10.168.0.5/Public/pub/nightlybuild/
 popd
 
