@@ -11,16 +11,19 @@ public:
 		std::string charset;
 	};
 private:
-	void *magic_set = nullptr;
-	std::vector<char> mgcdata;
+	void *magic_set_ = nullptr;
+	std::vector<char> mgcdata_;
 public:
+	bool open();
+#if 0
 	bool open(const char *mgcptr, size_t mgclen);
 	bool open(char const *mgcfile);
+#endif
 	void close();
 	static size_t slop_size();
 	Result file(int fd) const;
 	Result file(char const *filepath) const;
-	Result file(const char *data, size_t size, int filemode = 0644, bool pad_slop = true) const;
+	Result file(const char *data, size_t size, int filemode = 0644) const;
 };
 
 #endif // FILETYPE_H
