@@ -1,4 +1,6 @@
 #include "AbstractProcess.h"
+#include "ApplicationGlobal.h"
+#include "TraceEventWriter.h"
 
 void AbstractPtyProcess::setChangeDir(QString const &dir)
 {
