@@ -18,8 +18,8 @@ public:
 	explicit BigDiffWindow(QWidget *parent = nullptr);
 	~BigDiffWindow() override;
 
-	void init(BasicMainWindow *mw, const FileDiffWidget::InitParam_ &param);
-	void setTextCodec(QTextCodec *codec);
+	void init(const FileDiffWidget::InitParam_ &param);
+	void setTextCodec(std::shared_ptr<MyTextCodec> codec);
 private:
 	Ui::BigDiffWindow *ui;
 	void updateDiffView();
