@@ -16,7 +16,7 @@ class CommitDialog;
 class CommitDialog : public QDialog {
 	Q_OBJECT
 public:
-	explicit CommitDialog(MainWindow *parent, QString const &reponame, Git::User const &user, gpg::Data const &key, QString const &previousMessage);
+	explicit CommitDialog(MainWindow *parent, QString const &reponame, GitUser const &user, gpg::Data const &key, QString const &previousMessage);
 	~CommitDialog() override;
 
 	void setText(QString const &text);
