@@ -3,46 +3,35 @@
 
 #include <QDialog>
 #include "Git.h"
-#include "BasicMainWindow.h"
 #include "FileDiffWidget.h"
 
 namespace Ui {
 class FileHistoryWindow;
 }
 
-class BasicMainWindow;
 class QTableWidgetItem;
 
 class FileHistoryWindow : public QDialog {
 	Q_OBJECT
 private:
+	Ui::FileHistoryWindow *ui;
 	struct Private;
 	Private *m;
-
+	
+	MainWindow *mainwindow();
 	FileDiffWidget::DiffData *diffdata();
 	FileDiffWidget::DiffData const *diffdata() const;
-	FileDiffWidget::DrawData *drawdata();
-	FileDiffWidget::DrawData const *drawdata() const;
 	int totalTextLines() const;
-	int fileviewScrollPos() const;
-public:
-	explicit FileHistoryWindow(BasicMainWindow *parent);
-	~FileHistoryWindow() override;
-
-	void prepare(const GitPtr &g, QString const &path);
-private slots:
-	void on_tableWidget_log_currentItemChanged(QTableWidgetItem *current, QTableWidgetItem *previous);
-
-//	void onMoveNextItem();
-//	void onMovePreviousItem();
-	void on_tableWidget_log_customContextMenuRequested(const QPoint &pos);
-
-private:
-	Ui::FileHistoryWindow *ui;
-
 	void collectFileHistory();
 	void updateDiffView();
-	BasicMainWindow *mainwindow();
+public:
+	explicit FileHistoryWindow(MainWindow *parent);
+	~FileHistoryWindow() override;
+	
+	void prepare(GitRunner g, QString const &path);
+private slots:
+	void on_tableWidget_log_currentItemChanged(QTableWidgetItem *current, QTableWidgetItem *previous);
+	void on_tableWidget_log_customContextMenuRequested(const QPoint &pos);
 };
 
 #endif // FILEHISTORYWINDOW_H
