@@ -22,7 +22,7 @@ FilePropertyDialog::~FilePropertyDialog()
 	delete ui;
 }
 
-void FilePropertyDialog::exec(QString const &path, Git::Hash const &id)
+void FilePropertyDialog::exec(QString const &path, GitHash const &id)
 {
 	QFileInfo info(path);
 	if (!info.exists()) {
@@ -30,9 +30,16 @@ void FilePropertyDialog::exec(QString const &path, Git::Hash const &id)
 		return;
 	}
 	QByteArray ba;
-	QFile file(path);
-	if (file.open(QFile::ReadOnly)) {
-		ba = file.read(1024 * 1024);
+	if (id) {
+		GitObject obj = global->mainwindow->git().catFile(id);
+		if (obj.type == GitObject::Type::BLOB) {
+			ba = obj.content;
+		}
+	} else {
+		QFile file(path);
+		if (file.open(QFile::ReadOnly)) {
+			ba = file.read(1024 * 1024);
+		}
 	}
 	std::string mimetype = global->filetype.file(ba.data(), ba.size()).mimetype;
 	ui->lineEdit_repo->setText(global->mainwindow->currentRepositoryName());
