@@ -14,12 +14,12 @@ class MainWindow;
 class FilePropertyDialog : public QDialog {
 	Q_OBJECT
 private:
-	MainWindow *mainwindow;
+	// MainWindow *mainwindow;
 public:
 	explicit FilePropertyDialog(QWidget *parent = nullptr);
 	~FilePropertyDialog() override;
 
-	void exec(QString const &path, const Git::Hash &id);
+	void exec(QString const &path, const GitHash &id);
 private:
 	Ui::FilePropertyDialog *ui;
 };
