@@ -1,37 +1,34 @@
 #ifndef GITOBJECTMANAGER_H
 #define GITOBJECTMANAGER_H
 
-#include <QMutex>
-#include <QString>
-#include "GitPack.h"
-#include "GitPackIdxV2.h"
-#include <map>
-#include "common/joinpath.h"
-#include <mutex>
+#include "GitRunner.h"
 
 class GitPackIdxV2;
-
-class Git;
+struct GitPackIdxItem;
+struct GitPackObject;
 
 class GitObjectManager {
 	friend class GitObjectCache;
 private:
-	QString subdir_git_objects;
-	QString subdir_git_objects_pack;
-	std::vector<GitPackIdxPtr> git_idx_list;
+	struct Private;
+	Private *m;
 
 	static void applyDelta(QByteArray const *base, QByteArray const *delta, QByteArray *out);
-	static bool loadPackedObject(GitPackIdxPtr const &idx, QIODevice *packfile, GitPackIdxItem const *item, GitPack::Object *out);
-	bool extractObjectFromPackFile(GitPackIdxPtr const &idx, GitPackIdxItem const *item, GitPack::Object *out);
-	bool extractObjectFromPackFile(GitRunner g, const Git::Hash &id, QByteArray *out, Git::Object::Type *type);
-	void loadIndexes(GitRunner g);
-	QString findObjectPath(GitRunner g, const Git::Hash &id);
-	bool loadObject(GitRunner g, const Git::Hash &id, QByteArray *out, Git::Object::Type *type);
+	static bool loadPackedObject(std::shared_ptr<GitPackIdxV2> const &idx, QIODevice *packfile, GitPackIdxItem const *item, GitPackObject *out);
+	bool extractObjectFromPackFile(std::shared_ptr<GitPackIdxV2> const &idx, GitPackIdxItem const *item, GitPackObject *out);
+	bool extractObjectFromPackFile(GitRunner g, const GitHash &id, QByteArray *out, GitObject::Type *type, std::mutex *mutex);
+	void loadIndexes(GitRunner g, std::mutex *mutex);
+	QString findObjectPath(GitRunner g, const GitHash &id);
+	bool loadObject(GitRunner g, const GitHash &id, QByteArray *out, GitObject::Type *type);
 	void init();
 public:
-	GitObjectManager();
+	GitObjectManager(std::mutex *mutex);
+	GitObjectManager(GitObjectManager const &other);
+	GitObjectManager(GitObjectManager &&other) = delete;
+	~GitObjectManager();
+	GitObjectManager &operator = (GitObjectManager const &other);
 	void setup();
-	bool catFile(GitRunner g, const Git::Hash &id, QByteArray *out, Git::Object::Type *type);
+	bool catFile(GitRunner g, const GitHash &id, QByteArray *out, GitObject::Type *type);
 	void clearIndexes();
 
 	static QStringList findObject(const QString &id, const QString &repo_local_dir);
@@ -40,22 +37,27 @@ public:
 class GitObjectCache {
 public:
 	struct Item {
-		Git::Hash id;
+		GitHash id;
 		QByteArray ba;
-		Git::Object::Type type;
+		GitObject::Type type;
 	};
 private:
+	std::mutex *mutex_ = nullptr;
 	GitObjectManager object_manager_;
 	using ItemPtr = std::shared_ptr<Item>;
 	std::vector<ItemPtr> items_;
-	std::map<QString, Git::Hash> rev_parse_map_;
+	std::map<QString, GitHash> rev_parse_map_;
 	size_t size() const;
 public:
+	GitObjectCache(std::mutex *mutex = nullptr)
+		: mutex_(mutex)
+		, object_manager_(mutex)
+	{}
 	void clear();
-	Git::Hash revParse(GitRunner g, QString const &name);
-	Git::Object catFile(GitRunner g, const Git::Hash &id);
+	GitHash revParse(GitRunner g, QString const &name);
+	GitObject catFile(GitRunner g, const GitHash &id);
 
-	Git::Hash const &item_id(int i) const
+	GitHash const &item_id(int i) const
 	{
 		return items_[i]->id;
 	}
@@ -66,7 +68,7 @@ public:
 	QString tree_id;
 	QStringList parents;
 
-	static bool parseCommit(GitRunner g, GitObjectCache *objcache, Git::Hash const &id, GitCommit *out);
+	static bool parseCommit(GitRunner g, GitObjectCache *objcache, GitHash const &id, GitCommit *out);
 };
 
 struct GitTreeItem {
@@ -110,7 +112,7 @@ public:
 	bool lookup(GitRunner g, QString const &file, GitTreeItem *out);
 
 	void parseTree(GitRunner g, QString const &tree_id);
-	QString parseCommit(GitRunner g, const Git::Hash &commit_id);
+	QString parseCommit(GitRunner g, const GitHash &commit_id);
 
 	GitTreeItemList const *treelist() const
 	{
