@@ -1,16 +1,17 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include "RepositoryModel.h"
 #include "Git.h"
 #include "GitCommandRunner.h"
 #include "MyProcess.h"
 #include "RepositoryInfo.h"
+#include "RepositoryModel.h"
 #include "RepositoryTreeWidget.h"
+#include "StatusInfo.h"
 #include "TextEditorTheme.h"
 #include "UserEvent.h"
-#include "StatusInfo.h"
 #include <QMainWindow>
+#include <memory>
 
 class AddRepositoryDialog;
 class ApplicationSettings;
@@ -29,7 +30,7 @@ class MainWindow;
 }
 
 struct CloneParams {
-	Git::CloneData clonedata;
+	GitCloneData clonedata;
 	RepositoryInfo repodata;
 };
 Q_DECLARE_METATYPE(CloneParams)
@@ -74,11 +75,15 @@ class MainWindowExchangeData;
 
 class PtyProcessCompleted {
 public:
-	std::function<void (ProcessStatus const &, QVariant const &)> callback;
-	ProcessStatus status;
+	std::function<void (ProcessStatus *, QVariant const &)> callback;
+	std::shared_ptr<ProcessStatus> status;
 	QVariant userdata;
 	QString process_name;
 	QElapsedTimer elapsed;
+	PtyProcessCompleted()
+		: status(std::make_shared<ProcessStatus>())
+	{
+	}
 };
 Q_DECLARE_METATYPE(PtyProcessCompleted)
 
@@ -124,28 +129,12 @@ public:
 		SideBySide,
 	};
 
-	struct Task {
-		int index = 0;
-		int parent = 0;
-		Task() = default;
-		Task(int index, int parent)
-			: index(index)
-			, parent(parent)
-		{
-		}
-	};
-
-	struct Element {
-		int depth = 0;
-		std::vector<int> indexes;
-	};
-
 private:
 	struct ObjectData {
 		QString id;
 		QString path;
-		Git::SubmoduleItem submod;
-		Git::CommitItem submod_commit;
+		GitSubmoduleItem submod;
+		GitCommitItem submod_commit;
 		QString header;
 		int idiff;
 		bool staged = false;
@@ -156,12 +145,6 @@ private:
 	struct Private;
 	Private *m;
 
-	struct OpenRepositoyOption {
-		bool validate = false;
-		bool waitcursor = true;
-		bool keep_selection = false;
-	};
-
 	struct RepositoryTreeIndex {
 		int row = -1;
 	};
@@ -173,13 +156,13 @@ private:
 
 	class GitFile {
 	public:
-		Git::Object::Type type = Git::Object::Type::NONE;
+		GitObject::Type type = GitObject::Type::NONE;
 		QByteArray content;
 		operator bool () const
 		{
-			return type != Git::Object::Type::NONE;
+			return type != GitObject::Type::NONE;
 		}
-		bool is(Git::Object::Type t) const
+		bool is(GitObject::Type t) const
 		{
 			return t == type;
 		}
@@ -189,14 +172,28 @@ private:
 	void postUserEvent(UserEventHandler::variant_t &&v, int ms_later);
 	void cancelPendingUserEvents();
 
-	void updateFileList(const Git::Hash &id);
-	void updateFileList(const Git::CommitItem *commit);
+	void updateFileList(const GitHash &id);
+	void updateFileList(const GitCommitItem *commit);
 	void updateFileListLater(int delay_ms);
 	void cancelUpdateFileList();
 	void initUpdateFileListTimer();
 
-	void openRepositoryMain(GitRunner g, bool clear_log, bool do_fetch, bool keep_selection);
-	void doReopenRepository(const ProcessStatus &status, const RepositoryInfo &repodata);
+	struct OpenRepositoryOption {
+		bool new_session = true;
+
+		bool validate = false;
+		bool wait_cursor = true;
+		bool keep_selection = false;
+
+		bool clear_log = true;
+		bool do_fetch = true;
+	};
+	void openRepositoryMain(OpenRepositoryOption const &opt);
+	void openRepository(OpenRepositoryOption const &opt);
+	void reopenRepository(bool validate);
+	void openSelectedRepository();
+
+	void doReopenRepository(ProcessStatus *status, const RepositoryInfo &repodata);
 
 	QStringList selectedFiles_(QListWidget *listwidget) const;
 	QStringList selectedFiles() const;
@@ -227,11 +224,11 @@ private:
 	MainWindow::FilterTarget filtertarget() const;
 
 	void revertCommit();
-	void mergeBranch(const QString &commit, Git::MergeFastForward ff, bool squash);
-	void mergeBranch(Git::CommitItem const *commit, Git::MergeFastForward ff, bool squash);
-	void rebaseBranch(Git::CommitItem const *commit);
-	void cherrypick(Git::CommitItem const *commit);
-	void merge(Git::CommitItem commit = {});
+	void mergeBranch(const QString &commit, GitMergeFastForward ff, bool squash);
+	void mergeBranch(GitCommitItem const *commit, GitMergeFastForward ff, bool squash);
+	void rebaseBranch(GitCommitItem const *commit);
+	void cherrypick(GitCommitItem const *commit);
+	void merge(GitCommitItem commit = {});
 	void setRemoteOnline(bool f, bool save);
 	void startTimers();
 	void setNetworkingCommandsEnabled(bool enabled);
@@ -239,8 +236,8 @@ private:
 	void blame();
 	QListWidgetItem *currentFileItem() const;
 	void execAreYouSureYouWantToContinueConnectingDialog(const QString &windowtitle);
-	void deleteRemoteBranch(const Git::CommitItem &commit);
-	QStringList remoteBranches(const Git::Hash &id, QStringList *all);
+	void deleteRemoteBranch(const GitCommitItem &commit);
+	QStringList remoteBranches(const GitHash &id, QStringList *all);
 	bool isUninitialized();
 	void onLogCurrentItemChanged(bool update_file_list);
 	void findNext();
@@ -258,14 +255,12 @@ private:
 	void revertAllFiles();
 	bool execWelcomeWizardDialog();
 	void execRepositoryPropertyDialog(const RepositoryInfo &repo, bool open_repository_menu = false);
-	void execConfigUserDialog(const Git::User &global_user, const Git::User &local_user, bool enable_local_user, const QString &reponame);
+	void execConfigUserDialog(const GitUser &global_user, const GitUser &local_user, bool enable_local_user, const QString &reponame);
 	void setGitCommand(const QString &path, bool save);
 	void setGpgCommand(const QString &path, bool save);
 	void setSshCommand(const QString &path, bool save);
 	bool checkGitCommand();
-	bool saveBlobAs(const QString &id, const QString &dstpath);
 	bool saveByteArrayAs(const QByteArray &ba, const QString &dstpath);
-	bool saveFileAs(const QString &srcpath, const QString &dstpath);
 	QString executableOrEmpty(const QString &path);
 	bool checkExecutable(const QString &path);
 	void internalSaveCommandPath(const QString &path, bool save, const QString &name);
@@ -273,16 +268,13 @@ private:
 	void internalClearRepositoryInfo();
 	void checkUser();
 
-	void openRepository(OpenRepositoyOption const &opt);
-	void reopenRepository(bool validate);
-
 	void setCurrentRepository(const RepositoryInfo &repo, bool clear_authentication);
-	void openSelectedRepository();
-	std::optional<QList<Git::Diff> > makeDiffs(GitRunner g, Git::Hash id);
-	void updateRemoteInfo(GitRunner g);
-	void queryRemotes(GitRunner g);
+	std::optional<QList<GitDiff>> makeDiffs(GitRunner g, GitHash id, std::future<QList<GitSubmoduleItem>> &&async_modules);
+
+	void updateRemoteInfo();
+
 	void submodule_add(QString url = {}, const QString &local_dir = {});
-	const Git::CommitItem &selectedCommitItem() const;
+	const GitCommitItem &selectedCommitItem() const;
 	void commit(bool amend = false);
 	void commitAmend();
 	
@@ -291,15 +283,15 @@ private:
 	void push(bool set_upstream, const QString &remote, const QString &branch, bool force);
 	void fetch(GitRunner g, bool prune);
 	void stage(GitRunner g, const QStringList &paths);
-	void fetch_tags_f(GitRunner g);
+	void fetch(GitRunner g);
 	void pull(GitRunner g);
 	void push_tags(GitRunner g);
 	void delete_tags(GitRunner g, const QStringList &tagnames);
-	void add_tag(GitRunner g, const QString &name, Git::Hash const &commit_id);
+	void add_tag(GitRunner g, const QString &name, GitHash const &commit_id);
 
 	bool push();
 
-	void deleteBranch(const Git::CommitItem &commit);
+	void deleteBranch(const GitCommitItem &commit);
 	void deleteSelectedBranch();
 	void resetFile(const QStringList &paths);
 	void clearAuthentication();
@@ -311,25 +303,25 @@ private:
 	void addRepositoryAccepted(const AddRepositoryDialog &dlg);
 	void scanFolderAndRegister(const QString &group);
 	void doGitCommand(const std::function<void (GitRunner)> &callback);
-	void setWindowTitle_(const Git::User &user);
+	void setWindowTitle_(const GitUser &user);
 	void setUnknownRepositoryInfo();
 	void setCurrentRemoteName(const QString &name);
-	void deleteTags(const Git::CommitItem &commit);
+	void deleteTags(const GitCommitItem &commit);
 	QStringList remotes() const;
-	BranchList findBranch(const Git::Hash &id);
+	BranchList findBranch(const GitHash &id);
 	QString tempfileHeader() const;
 	void deleteTempFiles();
 	QString newTempFilePath();
 	int limitLogCount() const;
 	bool isThereUncommitedChanges() const;
-	Git::CommitItemList retrieveCommitLog(GitRunner g) const;
-	const std::map<Git::Hash, BranchList> &branchmap() const;
+	GitCommitItemList retrieveCommitLog(GitRunner g) const;
+	const std::map<GitHash, BranchList> &branchmap() const;
 
-	void updateWindowTitle(const Git::User &user);
+	void updateWindowTitle(const GitUser &user);
 	void updateWindowTitle(GitRunner g);
 
-	std::tuple<QString, BranchLabelList> makeCommitLabels(Git::CommitItem const &commit, std::map<Git::Hash, BranchList> const &branch_map, std::map<Git::Hash, TagList> const &tag_map) const;
-	QString labelsInfoText(Git::CommitItem const &commit);
+	std::tuple<QString, BranchLabelList> makeCommitLabels(GitCommitItem const &commit, std::map<GitHash, BranchList> const &branch_map, std::map<GitHash, TagList> const &tag_map) const;
+	QString labelsInfoText(GitCommitItem const &commit);
 
 	void removeRepositoryFromBookmark(RepositoryTreeIndex const &index, bool ask);
 	void openTerminal(const RepositoryInfo *repo);
@@ -337,18 +329,18 @@ private:
 	bool askAreYouSureYouWantToRun(const QString &title, const QString &command);
 	bool editFile(const QString &path, const QString &title);
 	void setAppSettings(const ApplicationSettings &appsettings);
-	QStringList findGitObject(const QString &id) const;
+
 	void saveApplicationSettings();
-	void loadApplicationSettings();
-	void setDiffResult(const QList<Git::Diff> &diffs);
-	const QList<Git::SubmoduleItem> &submodules() const;
-	void setSubmodules(const QList<Git::SubmoduleItem> &submodules);
+
+	void setDiffResult(const QList<GitDiff> &diffs);
+	const QList<GitSubmoduleItem> &submodules() const;
+	void setSubmodules(const QList<GitSubmoduleItem> &submodules);
 	bool runOnRepositoryDir(const std::function<void (QString, QString)> &callback, const RepositoryInfo *repo);
 	NamedCommitList namedCommitItems(int flags);
 	
-	const std::map<Git::Hash, TagList> &tagmap() const;
-	std::map<Git::Hash, TagList> queryTags(GitRunner g);
-	TagList findTag(const Git::Hash &id) const;
+	const std::map<GitHash, TagList> &tagmap() const;
+	std::map<GitHash, TagList> queryTags(GitRunner g);
+	TagList findTag(const GitHash &id) const;
 
 	void sshSetPassphrase(const std::string &user, const std::string &pass);
 	std::string sshPassphraseUser() const;
@@ -356,7 +348,7 @@ private:
 	void httpSetAuthentication(const std::string &user, const std::string &pass);
 	std::string httpAuthenticationUser() const;
 	std::string httpAuthenticationPass() const;
-	const Git::CommitItem *getLog(int index) const;
+	const GitCommitItem *getLog(int index) const;
 
 	bool saveRepositoryBookmarks();
 	QString getBookmarksFilePath() const;
@@ -377,45 +369,45 @@ private:
 	InteractionMode interactionMode() const;
 	void setInteractionMode(const InteractionMode &im);
 	void setUncommitedChanges(bool uncommited_changes);
-	QList<Git::Diff> const *diffResult() const;
+	QList<GitDiff> const *diffResult() const;
 
 	void clearLabelMap();
 	GitObjectCache *getObjCache();
 	bool getForceFetch() const;
 	void setForceFetch(bool force_fetch);
-	Git::Hash getHeadId() const;
-	void setHeadId(const Git::Hash &head_id);
+	GitHash getHeadId() const;
+	void setHeadId(const GitHash &head_id);
 	void setPtyProcessCompletionData(const QVariant &value);
 	const QVariant &getTempRepoForCloneCompleteV() const;
 	void msgNoRepositorySelected();
 	bool isRepositoryOpened() const;
-	void initRepository(const QString &path, const QString &reponame, const Git::Remote &remote);
+	void initRepository(const QString &path, const QString &reponame, const GitRemote &remote);
 	void updatePocessLog(bool processevents);
 	void appendLogHistory(const QByteArray &str);
 	std::vector<std::string> getLogHistoryLines();
 	void clearLogHistory();
-	void updateAvatar(const Git::User &user, bool request);
-	void cleanSubModule(QListWidgetItem *item);
+	void updateAvatar(const GitUser &user, bool request);
+	void cleanSubModule(GitRunner g, QListWidgetItem *item);
 
-	void updateUncommitedChanges();
+	void updateUncommitedChanges(GitRunner g);
 	void enableDragAndDropOnRepositoryTree(bool enabled);
 	QString preferredRepositoryGroup() const;
 	void setPreferredRepositoryGroup(const QString &group);
-	bool addExistingLocalRepository(QString dir, QString name, QString sshkey, bool open, bool save = true, bool msgbox_if_err = true);
+	bool _addExistingLocalRepository(QString dir, QString name, QString sshkey, bool open, bool save = true, bool msgbox_if_err = true);
 	void addExistingLocalRepositoryWithGroup(const QString &dir, const QString &group);
 	bool addExistingLocalRepository(const QString &dir, bool open);
 	QString currentFileMimeFileType() const;
 
-	int rowFromCommitId(const Git::Hash &id);
+	int rowFromCommitId(const GitHash &id);
 
 	void _updateCommitLogTableView(int delay_ms);
-	void makeCommitLog(CommitLogExchangeData exdata, int scroll_pos, int select_row);
+	void makeCommitLog(GitHash const &head, CommitLogExchangeData exdata, int scroll_pos, int select_row);
 
 	void updateButton();
-	void runPtyGit(const QString &progress_message, GitRunner g, GitCommandRunner::variant_t var, std::function<void (const ProcessStatus &, QVariant const &userdata)> callback, QVariant const &userdata);
+	void runPtyGit(const QString &progress_message, GitRunner g, GitCommandRunner::variant_t var, std::function<void (ProcessStatus *, QVariant const &userdata)> callback, QVariant const &userdata);
 	CommitLogExchangeData queryCommitLog(GitRunner g);
 	void updateHEAD(GitRunner g);
-	bool jump(GitRunner g, const Git::Hash &id);
+	bool jump(GitRunner g, const GitHash &id);
 	void jump(GitRunner g, const QString &text);
 	void connectPtyProcessCompleted();
 	void setupShowFileListHandler();
@@ -424,8 +416,11 @@ private:
 	void clearRetry();
 	void retry();
 	bool isRetryQueued() const;
+
 	void clearGitCommandCache();
-	Git::CommitItemList log_all2(GitRunner g, const Git::Hash &id, int maxcount) const;
+	void clearGitObjectCache();
+
+	GitCommitItemList log_all2(GitRunner g, const GitHash &id, int maxcount) const;
 	ProgressWidget *progress_widget() const;
 	void internalShowPanel(FileListType file_list_type);
 	
@@ -438,6 +433,9 @@ private:
 
 	void _chooseRepository(QTreeWidgetItem *item);
 	void chooseRepository();
+	void setCurrentGitRunner(GitRunner g);
+	void endSession();
+	std::optional<GitCommitItem> getCommitItem(GitRunner g, const GitHash &hash) const;
 protected:
 	void closeEvent(QCloseEvent *event) override;
 	void dragEnterEvent(QDragEnterEvent *event) override;
@@ -554,6 +552,10 @@ private slots:
 
 	void on_action_view_sort_by_time_changed();
 
+	void on_action_ssh_triggered();
+
+	void on_action_restart_trace_logger_triggered();
+
 signals:
 	void signalUpdateCommitLog();
 	void signalSetProgress(float progress);
@@ -580,17 +582,14 @@ public:
 	
 	const TagList &queryCurrentCommitTagList() const;
 	
-	// static bool isGroupItem(QTreeWidgetItem *item);
-	// static void setRepoIndex(QTreeWidgetItem *item, int index);
-	// static int repoIndex(QTreeWidgetItem *item);
 	static int indexOfLog(QListWidgetItem *item);
 	static int indexOfDiff(QListWidgetItem *item);
-	static void updateSubmodules(GitRunner g, const Git::Hash &id, QList<Git::SubmoduleItem> *out);
-	static void updateCommitGraph(Git::CommitItemList *logs);
-	static TagList findTag(std::map<Git::Hash, TagList> const &tagmap, Git::Hash const &id);
+	static QList<GitSubmoduleItem> updateSubmodules(GitRunner g, const GitHash &id);
+	static void updateCommitGraph(GitCommitItemList *logs);
+	static TagList findTag(std::map<GitHash, TagList> const &tagmap, GitHash const &id);
 	static QString makeRepositoryName(const QString &loc);
-	static void addDiffItems(const QList<Git::Diff> *diff_list, const std::function<void (const ObjectData &)> &add_item);
-	static Git::Hash getObjectID(QListWidgetItem *item);
+	static void addDiffItems(const QList<GitDiff> *diff_list, const std::function<void (const ObjectData &)> &add_item);
+	static GitHash getObjectID(QListWidgetItem *item);
 	static QString getFilePath(QListWidgetItem *item);
 	static QString getSubmodulePath(QListWidgetItem *item);
 	static QString getSubmoduleCommitId(QListWidgetItem *item);
@@ -600,7 +599,8 @@ public:
 	// static QString treeItemName(QTreeWidgetItem *item);
 	// static QString treeItemGroup(QTreeWidgetItem *item);
 	static bool isValidWorkingCopy(GitRunner g);
-	static QString abbrevCommitID(const Git::CommitItem &commit);
+	bool isValidWorkingCopy(QString const &local_dir);
+	static QString abbrevCommitID(const GitCommitItem &commit);
 
 	void drawDigit(QPainter *pr, int x, int y, int n) const;
 	static constexpr int DIGIT_WIDTH = 5;
@@ -613,7 +613,7 @@ public:
 	void addTag(QString const &name);
 	int selectedLogIndex() const;
 	void updateAncestorCommitMap();
-	bool isAncestorCommit(const QString &id);
+	bool isAncestorCommit(GitHash const &id) const;
 	void postStartEvent(int ms_later);
 	void setShowLabels(bool show, bool save);
 	void setShowGraph(bool show, bool save);
@@ -622,10 +622,10 @@ public:
 	bool isGraphVisible() const;
 	bool isAvatarsVisible() const;
 	bool isAvatarEnabled() const;
-	void makeDiffList(const Git::Hash &id, QList<Git::Diff> *diff_list, QListWidget *listwidget);
-	void execCommitViewWindow(const Git::CommitItem *commit);
-	void execCommitPropertyDialog(QWidget *parent, const Git::CommitItem &commit);
-	void execCommitExploreWindow(QWidget *parent, const Git::CommitItem *commit);
+	void makeDiffList(const GitHash &id, QList<GitDiff> *diff_list, QListWidget *listwidget);
+	void execCommitViewWindow(const GitCommitItem *commit);
+	void execCommitPropertyDialog(QWidget *parent, const GitCommitItem &commit);
+	void execCommitExploreWindow(QWidget *parent, const GitCommitItem *commit);
 	void execFileHistory(const QString &path);
 	void execFileHistory(QListWidgetItem *item);
 	void showObjectProperty(QListWidgetItem *item);
@@ -633,45 +633,44 @@ public:
 	QString selectGitCommand(bool save);
 	QString selectGpgCommand(bool save);
 	QString selectSshCommand(bool save);
-	const Git::Branch &currentBranch() const;
-	void setCurrentBranch(const Git::Branch &b);
+	const GitBranch &currentBranch() const;
+	void setCurrentBranch(const GitBranch &b);
 	const RepositoryInfo &currentRepository() const;
 	QString currentRepositoryName() const;
 	QString currentRemoteName() const;
 	QString currentBranchName() const;
-	GitRunner git(const QString &dir, const QString &submodpath, const QString &sshkey, bool use_cache = true) const;
+	GitRunner _git(const QString &dir, const QString &submodpath, const QString &sshkey) const;
+	GitRunner unassosiated_git_runner() const;
+	GitRunner new_git_runner(const QString &dir, const QString &sshkey);
+	GitRunner new_git_runner();
 	GitRunner git();
-	GitRunner git(Git::SubmoduleItem const &submod);
+	static GitRunner git_for_submodule(GitRunner g, GitSubmoduleItem const &submod);
+	bool isValidWorkingCopy(QString const &dir) const;
 	void autoOpenRepository(QString dir, const QString &commit_id = {});
-	std::optional<Git::CommitItem> queryCommit(const Git::Hash &id);
+	std::optional<GitCommitItem> queryCommit(const GitHash &id);
 	bool checkoutLocalBranch(QString const &name);
-	void checkout(QWidget *parent, const Git::CommitItem &commit, std::function<void ()> accepted_callback = {});
+	void checkout(QWidget *parent, const GitCommitItem &commit, std::function<void ()> accepted_callback = {});
 	void checkout();
-	bool jumpToCommit(const Git::Hash &id);
+	bool jumpToCommit(const GitHash &id);
 	bool jumpToCommit(const QString &id);
 
-	Git::Object catFile(GitRunner g, const QString &id);
-	bool saveAs(const QString &id, const QString &dstpath);
-	static std::string determineFileType(QByteArray const &in);
-        static std::string determineFileType(const std::string &path);
-
 	TextEditorThemePtr themeForTextEditor();
 	void emitWriteLog(LogData const &logdata);
-	QString findFileID(const Git::Hash &commit_id, const QString &file);
-	const Git::CommitItem &commitItem(int row) const;
-	const Git::CommitItem &commitItem(Git::Hash const &id) const;
+	QString findFileID(const GitHash &commit_id, const QString &file);
+	const GitCommitItem &commitItem(int row) const;
+	const GitCommitItem &commitItem(GitHash const &id) const;
 	QImage committerIcon(int row, QSize size) const;
 	void changeSshKey(const QString &local_dir, const QString &ssh_key, bool save);
 	ApplicationSettings *appsettings();
 	const ApplicationSettings *appsettings() const;
 	QString defaultWorkingDir() const;
-	QIcon signatureVerificationIcon(const Git::Hash &id) const;
+	QIcon signatureVerificationIcon(const GitHash &id) const;
 	QAction *addMenuActionProperty(QMenu *menu);
 	QString currentWorkingCopyDir() const;
-	Git::SubmoduleItem const *querySubmoduleByPath(const QString &path, Git::CommitItem *commit);
+
 	void refresh();
-	bool cloneRepository(const Git::CloneData &clonedata, const RepositoryInfo &repodata);
-	Git::User currentGitUser() const;
+	bool cloneRepository(const GitCloneData &clonedata, const RepositoryInfo &repodata);
+	GitUser currentGitUser() const;
 	void setupExternalPrograms();
 	void updateCommitLogTableViewLater();
 	void saveRepositoryBookmark(RepositoryInfo item);
@@ -682,8 +681,8 @@ public:
 	void hideProgress();
 	void internalAfterFetch();
 	void onRepositoryTreeSortRecent(bool f);
-	const Git::CommitItemList &commitlog() const;
-	const Git::CommitItem *currentCommitItem();
+	const GitCommitItemList &commitlog() const;
+	const GitCommitItem *currentCommitItem();
 	void clearLogContents();
 	void updateLogTableView();
 	void setFocusToLogTable();
@@ -691,6 +690,8 @@ public:
 	RepositoryData *currentRepositoryData();
 	const RepositoryData *currentRepositoryData() const;
 	void setCommitLog(const CommitLogExchangeData &exdata);
+	bool saveFileAs(const QString &srcpath, const QString &dstpath);
+	bool saveBlobAs(const QString &id, const QString &dstpath);
 public slots:
 	void internalWriteLog(const LogData &logdata);
 };
