@@ -19,7 +19,7 @@ public:
 	QString getFastForwardPolicy() const;
 	void setFastForwardPolicy(const QString &ff);
 	QString mergeFrom() const;
-	static Git::MergeFastForward ff(const QString &ff);
+	static GitMergeFastForward ff(const QString &ff);
 	bool isSquashEnabled() const;
 private slots:
 	void on_listWidget_from_itemDoubleClicked(QListWidgetItem *item);
