@@ -8,42 +8,30 @@
 MyImageViewWidget::MyImageViewWidget(QWidget *parent)
 	: ImageViewWidget(parent)
 {
-
 }
 
 void MyImageViewWidget::setImage(std::string const &mimetype, QByteArray const &ba, QString const &object_id, QString const &path)
 {
 	this->object_id_ = object_id;
-	this->path_ = path;
+	this->object_path_ = path;
 	ImageViewWidget::setImage(mimetype, ba);
 }
 
 void MyImageViewWidget::contextMenuEvent(QContextMenuEvent *e)
 {
-	QString id = object_id_;
-	if (id.startsWith(PATH_PREFIX)) {
-		// pass
-	} else if (Git::isValidID(id)) {
-		// pass
-	} else {
-		return; // invalid id
-	}
+	if (!isValidObject()) return;
 
 	QMenu menu;
 
-	QAction *a_save_as = id.isEmpty() ? nullptr : menu.addAction(tr("Save as..."));
+	QAction *a_save_as = menu.addAction(tr("Save as..."));
 	if (!menu.actions().isEmpty()) {
 		update();
 		QAction *a = menu.exec(misc::contextMenuPos(this, e));
 		if (a) {
 			if (a == a_save_as) {
-				auto *mw = qobject_cast<MainWindow *>(mainwindow());
-				QString path = mw->currentWorkingCopyDir() / path_;
-				QString dstpath = QFileDialog::getSaveFileName(window(), tr("Save as"), path);
-				if (!dstpath.isEmpty()) {
-					mw->saveAs(id, dstpath);
+				if (saveAs(window())) {
+					update();
 				}
-				update();
 				return;
 			}
 		}
