@@ -2,16 +2,14 @@
 #define MYIMAGEVIEWWIDGET_H
 
 #include "ImageViewWidget.h"
+#include "MyObjectViewBase.h"
 
 // used from FileViewWidget.h
 
 // ImageViewWidgetクラスは、他のアプリで再利用する想定の設計。
 // MyImageViewWidgetクラスは、Guitar専用にカスタマイズを行っている。
 
-class MyImageViewWidget : public ImageViewWidget {
-private:
-	QString object_id_;
-	QString path_;
+class MyImageViewWidget : public MyObjectViewBase, public ImageViewWidget {
 protected:
 	void contextMenuEvent(QContextMenuEvent *e) override;
 public:
