@@ -1,6 +1,7 @@
 #ifndef PROFILE_H
 #define PROFILE_H
 
+#include "TraceLogger.h"
 #include <QElapsedTimer>
 #include <QString>
 
@@ -8,6 +9,7 @@ class Profile {
 private:
 	QString func_;
 	QElapsedTimer timer_;
+	TraceLogger trace_logger_;
 	void log(QString const &s);
 public:
 	Profile(const char *func);
