@@ -30,7 +30,8 @@ using socket_t = int;
 #include <openssl/evp.h>
 #include <openssl/rand.h>
 #include <openssl/x509v3.h>
-#if (OPENSSL_VERSION_NUMBER >= 0x30000000L && !defined SSL_get_peer_certificate)
+// #if (OPENSSL_VERSION_NUMBER >= 0x30000000L && !defined SSL_get_peer_certificate)
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
 #define SSL_get_peer_certificate(s) SSL_get1_peer_certificate(s)
 #endif
 #else
@@ -102,7 +103,7 @@ bool HostNameResolver::resolve(const char *name, _in_addr *out)
 	if (!name || !out) return false;
 
 	struct hostent *he = nullptr;
-#if defined(_WIN32) || defined(__APPLE__)
+#if defined(_WIN32) || defined(__APPLE__) || defined(__NetBSD__)
 	he = ::gethostbyname(name);
 #else
 	int err = 0;
